@@ -20,4 +20,4 @@ release.upload_asset('singleheader/ada.cpp')
 release.upload_asset('singleheader/ada.h')
 release.upload_asset('singleheader/ada_c.h')
 release.upload_asset('singleheader/singleheader.zip')
-release.update_release(draft=False)
+release.update_release(name=release.name, message=release.body, draft=False, make_latest="true")
